/**
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 2009 Bull S.A.S.
 * Contact: jonas-team@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id$
 * --------------------------------------------------------------------------
 */

package org.ow2.jonas.admin.auditconsole.service;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.ow2.util.auditreport.impl.GenericAuditReport;
import org.ow2.util.auditreport.impl.InvocationAuditReport;
import org.ow2.util.auditreport.impl.JNDIAuditReport;
import org.ow2.util.auditreport.impl.WebInvocationAuditReport;

/**

 * @author Mathieu Ancelin
 */
public class CallTops implements Serializable {

    /**
     * Serial Version UID.
     */
    private static final long serialVersionUID = -9130174828112629672L;

    private int topsNumber = 5;

    private Map<String, TopMethod> globalTop = new HashMap<String, TopMethod>();

    private Map<String, TopMethod> ejbTop = new HashMap<String, TopMethod>();

    private Map<String, TopMethod> webTop = new HashMap<String, TopMethod>();

    private Map<String, TopMethod> jndiTop = new HashMap<String, TopMethod>();

    public CallTops() {
    }

    public void addReport(final GenericAuditReport report) {
        if (report instanceof InvocationAuditReport) {
            incrementTop(ejbTop, ((InvocationAuditReport) report).getTarget());
            incrementTop(globalTop, ((InvocationAuditReport) report).getTarget());
        } else if (report instanceof WebInvocationAuditReport) {
            incrementTop(webTop, ((WebInvocationAuditReport) report).getRequestURL() + "/"
                + ((WebInvocationAuditReport) report).getBusinessMethod());
            incrementTop(globalTop, ((WebInvocationAuditReport) report).getRequestURL() + "/"
                + ((WebInvocationAuditReport) report).getBusinessMethod());
        } else if (report instanceof JNDIAuditReport) {
            incrementTop(jndiTop, ((JNDIAuditReport) report).getTarget());
            incrementTop(globalTop, ((JNDIAuditReport) report).getTarget());
        }
    }

    public void incrementTop(final Map<String, TopMethod> map, final String target) {
        String[] parts = target.split("/");
        int size = parts.length;
        String name = "";
        if (size >= 2) {
            name += parts[size - 2] + "/" + parts[size - 1];
        } else if (size == 1) {
            name = parts[0];
        } else {
            name = target;
        }
        if (!map.containsKey(target)) {
            TopMethod top = new TopMethod(target, name, 1);
            map.put(target, top);
        } else {
            TopMethod top = map.get(target);
            top.setCount(top.getCount() + 1);
        }
    }

    public List<TopMethod> getGlobalTop() {
        ArrayList<TopMethod> tops = new ArrayList<TopMethod>(globalTop.values());
        Collections.sort(tops);
        if (tops.size() >= (topsNumber - 1)) {
            return tops.subList(0, topsNumber - 1);
        } else {
            return tops;
        }
    }

    public List<TopMethod> getEjbTop() {
        ArrayList<TopMethod> tops = new ArrayList<TopMethod>(ejbTop.values());
        Collections.sort(tops);
        if (tops.size() >= (topsNumber - 1)) {
            return tops.subList(0, topsNumber - 1);
        } else {
            return tops;
        }
    }

    public List<TopMethod> getWebTop() {
        ArrayList<TopMethod> tops = new ArrayList<TopMethod>(webTop.values());
        Collections.sort(tops);
        if (tops.size() >= (topsNumber - 1)) {
            return tops.subList(0, topsNumber - 1);
        } else {
            return tops;
        }
    }

    public List<TopMethod> getJndiTop() {
        ArrayList<TopMethod> tops = new ArrayList(jndiTop.values());
        Collections.sort(tops);
        if (tops.size() >= (topsNumber - 1)) {
            return tops.subList(0, topsNumber - 1);
        } else {
            return tops;
        }
    }

    public int getTopsNumber() {
        return topsNumber;
    }

    public void setTopsNumber(final int topsNumber) {
        this.topsNumber = topsNumber;
    }
}
